#
# fbuilder.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@loveruby.net>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

module ReFe

  class FunctionIndexBuilder

    def self.build( fname, index )
      index.def_updating {
        new.build fname, index
      }
    end

    def build( fname, index )
      File.open( fname ) {|f|
        _build f, index, fname
      }
    end

    def _build( f, index, fname )
      pre = nil
      while line = f.gets do
        case line
        when /\A\w/
          (pre ||= []).push line
        when /\A\s*\z/
          pre = nil
        when /\A\{/
begin
          if pre then
            index.register_definition get_fname(pre),
                                      pre.join(''), read_function(f, line),
                                      "#{fname}:#{f.lineno}"
          end
rescue
  puts '---- error'
  p f.lineno
  p pre.type
  p pre
  raise
end
          pre = nil
        else
          pre.push line if pre
        end
      end
    end

    def get_fname( pre )
      m = /\A([^(]+)/.match( pre[1] ) or
      m = /(\w+)\s*\(/.match( pre[0] ) or
              raise ArgumentError, "cannot get func name: #{pre[1]}"
      m[1]
    end

    def read_function( f, line )
      ret = [line]
      while line = f.gets do
        ret.push line
        break if /\A\};?\s*\z/ === line
      end
      ret.join('')
    end
  
  end   # class FunctionIndexBuilder

end
